/* reverse-comp.c -- compute the reverse complement of a sequence
 *
 * This program reads a sequence file in FastA format and produces
 * an output file containing the reverse complement of the given
 * sequence, again in FastA format.  A notation saying "(reverse
 * complement)" will be added to (or removed from) the end of the
 * FastA header line.  If the file contains multiple sequences
 * (i.e., contigs), then each of them is reversed individually.
 *
 * $Revision: 1.7 $$Date: 2001/04/04 21:40:30 $
 */

#include "seq.h"
#include "util.h"

#define SYNTAX  "seq_file > output_file"
#define REV     " (reverse complement)"

static void get_args (int argc, char **argv, char **seqfile);
static void adjust_hdr (SEQ *sp);

int main (int argc, char **argv) {
	char *seqfile = NULL;                   /* name of input file           */
	SEQ *sp;                                /* ptr to struct holding seq    */
	int width;                              /* number of columns to print   */

	/* get arguments */
	set_argv0 (argv[0]);
	get_args (argc, argv, &seqfile);
	if (!seqfile)
		fatal (usage (SYNTAX));

	/* open sequence file */
	sp = seq_open (seqfile);
	if (SEQ_FROM (sp) != 1)
		fatal ("Specification of subsequence is not supported.");

	/* loop over contigs */
	width = seq_get_width (sp);
	while (seq_read (sp) > 0) {
		do_revcomp (SEQ_CHARS (sp), SEQ_LEN (sp));    /* in place */
		adjust_hdr (sp);
		seq_print (sp, width);
		putchar ('\n');
		width = seq_get_width (sp);
	}
	if (!sp->count)
		fatalf ("Couldn't get sequence from \"%s\".", seqfile);

	/* clean up */
	seq_close (sp);
	return 0;
}

/* retrieve command-line arguments */
static void get_args (int argc, char **argv, char **seqfile) {
	int i;

	for (i = 1; i < argc; i++) {
		if (!*seqfile)
			*seqfile = argv[i];
		else
			fatal (usage (SYNTAX));
	}
}

/* add or remove reverse complement indicator */
static void adjust_hdr (SEQ *sp) {
	char *hdr, *hdr2, *p;

	hdr = sp->header;
	if ((p = strstr (hdr, REV)))
		*p = '\0';
	else {
		hdr2 = (char *) ckalloc (strlen (hdr) + strlen (REV) + 1);
		strcpy (hdr2, hdr);
		strcat (hdr2, REV);
		free (sp->header);
		sp->header = hdr2;
		sp->hlen = strlen (hdr2);
	}
}
